/* array.h - Array operations

   Copyright (c) 2010 Jukka Lehtosalo

   Alore is licensed under the terms of the MIT license.  See the file
   LICENSE.txt in the distribution.
*/

#ifndef ARRAY_H_INCL
#define ARRAY_H_INCL

#include "thread.h"
#include "operator.h"


AValue AArrayCreate(AThread *t, AValue *frame);
AValue AArrayInitialize(AThread *t, AValue *frame);
AValue AArray_get(AThread *t, AValue *frame);
AValue AArray_set(AThread *t, AValue *frame);
AValue AArray_in(AThread *t, AValue *frame);
AValue AArrayLengthMethod(AThread *t, AValue *frame);
AValue AArray_add(AThread *t, AValue *frame);
AValue AArray_mul(AThread *t, AValue *frame);
AValue AArray_eq(AThread *t, AValue *frame);
AValue AArray_lt(AThread *t, AValue *frame);
AValue AArray_gt(AThread *t, AValue *frame);
AValue AArrayAppend(AThread *t, AValue *frame);
AValue AArrayInsertAt(AThread *t, AValue *frame);
AValue AArrayRemoveAt(AThread *t, AValue *frame);
AValue AArray_str(AThread *t, AValue *frame);
AValue AArrayIter(AThread *t, AValue *frame);
AValue AArrayFind(AThread *t, AValue *frame);
AValue AArrayIndex(AThread *t, AValue *frame);
AValue AArrayCount(AThread *t, AValue *frame);
AValue AArray_hash(AThread *t, AValue *frame);
AValue AArrayCopy(AThread *t, AValue *frame);

AValue AArrayIterCreate(AThread *t, AValue *frame);
AValue AArrayIterHasNext(AThread *t, AValue *frame);
AValue AArrayIterNext(AThread *t, AValue *frame);


extern int AArrayIterNum;


AValue AMakeArrayND(AThread *t, Assize_t length);
#define AMakeUninitArray_M(t, length, array, result) \
    do {                              \
        unsigned long instSize, size; \
        AInstance *inst;              \
        AFixArray *fixArray;          \
                                      \
        instSize = AArrayClass->instanceSize; \
        size = instSize + AGetBlockSize(((length) + 1) * sizeof(AValue)); \
        AAlloc_M(t, size, inst, (result));  \
        if (!(result)) {                    \
            ARaisePreallocatedMemoryErrorND(t); \
            break;                          \
        }                                   \
        fixArray = APtrAdd(inst, instSize); \
                                            \
        AInitInstanceBlock(&inst->type, AArrayClass);         \
        inst->member[A_ARRAY_LEN] = AIntToValue(length);      \
        inst->member[A_ARRAY_CAPACITY] = AIntToValue(length); \
        inst->member[A_ARRAY_A] = AFixArrayToValue(fixArray); \
        AInitValueBlock(&fixArray->header, (length) * sizeof(AValue)); \
        (array) = AInstanceToValue(inst); \
    } while (0)
ABool ASetArrayItemND(AThread *t, AValue arrVal, Assize_t index, AValue item);
#define ASetArrayItemNewGen(a, index, v) \
    (AArrayItem(a, index) = (v))


AValue AMakeFixArray(AThread *t, Assize_t length, AValue init);

AValue AConcatArrays(AThread *t, AValue arrVal1, AValue arrVal2);
AValue ASubArrayND(AThread *t, AValue arrVal, Assize_t begInd,
                   Assize_t endInd);
AValue ACompareArrays(AThread *t, AValue a1, AValue a2, AOperator operator);
AValue AArrayHashValue(AThread *t, AValue *frame);

void AResizeArray(AThread *t, AValue *frame);


#endif
